unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    ControlPanel: TPanel;
    ConfigButton: TButton;
    CalLabel: TLabel;
    CalEdit: TComboBox;
    CalButton: TButton;
    EnableButton: TButton;
    DisableButton: TButton;
    GetADButton: TButton;
    LogMemo: TMemo;
    Splitter1: TSplitter;
    Get8Button: TButton;
    Button1: TButton;
    ButtonFineUp: TButton;
    ButtonPhaseCoarseUp: TButton;
    ButtonCoarseDown: TButton;
    ButtonFineDown: TButton;
    ButtonPhaseReset: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure ConfigButtonClick(Sender: TObject);
    procedure CalButtonClick(Sender: TObject);
    procedure EnableButtonClick(Sender: TObject);
    procedure DisableButtonClick(Sender: TObject);
    procedure GetADButtonClick(Sender: TObject);
    procedure Get8ButtonClick(Sender: TObject);
    procedure Get4ButtonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure PhaseAdjust(Sender: TObject);
  private
  protected
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOUSB, DetectUnit;

const
  HexChar: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

type
  TUSBAI1616Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //added for compatibility with the AI-MUX family
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI1616Config;
  Counts: array[0..15] of Word;
  Volts: array[0..15] of Double;
  PhaseOffset : integer;

function NybbleToHex(N: Byte): Char;
begin
  Result := HexChar[N and $F];
end;

function StrToSpacedHex(Text: String): String;
var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := HexChar[Byte(Text[I]) shr 4];
    Result[I * 3 - 1] := HexChar[Byte(Text[I]) and $F];
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  DetectForm: TDetectForm;
  Status, PID: LongWord;
  bCanCal: Boolean;
  iChannel: Integer;

begin
  for iChannel := 0 to 15 do Counts[iChannel] := $8000;
  for iChannel := 0 to 15 do Volts[iChannel] := 4;//9001;

  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or not (((PID >= $8040) and (PID <= $8044)) or ((PID >= $8140) and (PID <= $8144))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  bCanCal := ADC_QueryCal(DeviceIndex) = ERROR_SUCCESS;
  CalLabel.Enabled := bCanCal;
  CalEdit.Enabled := bCanCal;
  CalButton.Enabled := bCanCal;
end;

procedure TMainForm.FormPaint(Sender: TObject);
var
  iChannel, W: Integer;
  F: Double;
  R: TRect;
  Sz: TSize;
  Buf: String;

begin
  W := Max(0, ControlPanel.Left - 16);
  R.Left := 8;
  for iChannel := 0 to 15 do begin
    if Volts[iChannel] > 9000 then Continue;

    Canvas.Brush.Color := $000000;
    R.Top := 8 + 24 * iChannel;
    R.Bottom := R.Top + 16;
    R.Right := 8 + W;
    Canvas.FillRect(R);

    Canvas.Brush.Color := $FFFFFF;
    F := 1/$10000 * W * Counts[iChannel];
    R.Right := Round(8 + F);
    Canvas.FillRect(R);

    Canvas.Brush.Style := bsClear;
    Canvas.Font.Color := $FF0000;
    Buf := IntToHex(Counts[iChannel], 4) + ' / ' + FloatToStr(Volts[iChannel]);
    Sz := Canvas.TextExtent(Buf);
    Canvas.TextOut(8 + ((W - Sz.cx) div 2), R.Top + ((16 - Sz.cy) div 2), Buf);
  end;
end;

procedure TMainForm.ConfigButtonClick(Sender: TObject);
var
  ConfigBufSize: LongWord;
  I: Integer;
  RangeCode: Byte;
  
begin
  RangeCode := $03;
  //if DiffCheck.Checked then RangeCode := RangeCode or $08;
  for I := $0 to $F do
    Config.ChannelRange[I] := RangeCode
  ;
  //if DiffCheck.Checked then Config.StartStopCH := $70 else //Select all 8 differential channels, from 0 to 7.
  Config.StartStopCH := $F0; //Select all 16 channels, from 0 to 15.
  Config.CalibMode := 0; //Take actual data, not internal calibration sources.
  Config.TrigMode := $05; //Scan selected channels each counter rising edge.
  Config.Oversample := 0; //No oversample.

  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.CalButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  DisableButtonClick(self);
  Status := ADC_SetCal(DeviceIndex, PChar(CalEdit.Text));
  if Status <> ERROR_SUCCESS then LogMemo.Lines.Add('Calibration Error ' + IntToStr(Status))
  else LogMemo.Lines.Add('Calibrated');
end;

procedure TMainForm.EnableButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  Status := GenericVendorWrite(DeviceIndex, $C2, 0, $0001, 0, nil);
  if Status <> ERROR_SUCCESS then LogMemo.Lines.Add('C2 Write Error ' + IntToStr(Status));
end;

procedure TMainForm.DisableButtonClick(Sender: TObject);
var
  Status: LongWord;

begin
  Status := GenericVendorWrite(DeviceIndex, $C2, 0, $0000, 0, nil);
  if Status <> ERROR_SUCCESS then LogMemo.Lines.Add('C2 Write Error ' + IntToStr(Status))
  else LogMemo.Lines.Add('Disabled');
end;

procedure TMainForm.GetADButtonClick(Sender: TObject);
var
  Status, BufSize: LongWord;
  iChannel: Integer;
  ADData: array[0..15] of Word;
  V: Double;

begin
  BufSize := SizeOf(ADData);
  Status := GenericVendorRead(DeviceIndex, $C2, 0, 0, BufSize, @ADData[0]);
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('C2 Read Error ' + IntToStr(Status));
    Exit;
  end;

  for iChannel := 0 to 15 do begin
    Counts[iChannel] := ADData[iChannel];
    V := 1/$10000 * ADData[iChannel];
    V := V * 2 - 1;
    V := V * 5;
    //V := V * 2;
    Volts[iChannel] := V;
  end;

  FormPaint(Self);
end;

procedure TMainForm.Get8ButtonClick(Sender: TObject);
var
  Status, BufSize: LongWord;
  Buf: String;

begin
  BufSize := 16;
  SetLength(Buf, BufSize);
  Status := GenericVendorRead(DeviceIndex, $C2, 0, 0, BufSize, PChar(Buf));
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('C2 Read Error ' + IntToStr(Status));
    Exit;
  end;

  LogMemo.Lines.Add(StrToSpacedHex(Buf));
end;

procedure TMainForm.Get4ButtonClick(Sender: TObject);
var
  Status, BufSize: LongWord;
  Buf: String;

begin
  BufSize := 4;
  SetLength(Buf, BufSize);
  Status := GenericVendorRead(DeviceIndex, $C2, 0, 0, BufSize, PChar(Buf));
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('C2 Read Error ' + IntToStr(Status));
    Exit;
  end;

  LogMemo.Lines.Add(StrToSpacedHex(Buf));
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  Status, BufSize: LongWord;
  Buf: String;

begin
  Status := GenericVendorWrite(DeviceIndex, $C2, 0, $0001, 0, nil);
  if Status <> ERROR_SUCCESS then LogMemo.Lines.Add('C2 Write Error ' + IntToStr(Status));

  BufSize := 4;
  SetLength(Buf, BufSize);
  Status := GenericVendorRead(DeviceIndex, $C2, 0, 1, BufSize, PChar(Buf));
  if Status <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('C2 Read Error ' + IntToStr(Status));
    Exit;
  end;

  LogMemo.Lines.Add('Calibration Results: ' + StrToSpacedHex(Buf));
end;

procedure TMainForm.PhaseAdjust(Sender: TObject);
var
  Status: LongWord;
  buf:byte;
begin

  if (Sender as TButton).Tag = 0 then PhaseOffset := 0
  else
    inc( PhaseOffset, (Sender as TButton).Tag );
  if PhaseOffset < 0 then
      PhaseOffset :=0;
  if PhaseOffset > 999 then
      PhaseOffset := 999;

  buf := PhaseOffset mod 100;
  Status := GenericVendorWrite(DeviceIndex, $C2, 0, PhaseOffset div 100, 1, @buf);
  if Status <> ERROR_SUCCESS then LogMemo.Lines.Add('C2 Write Error ' + IntToStr(Status))
  else LogMemo.Lines.Add('Phase adjust set to ' + IntToStr(PhaseOffset) + '%');

end;

end.
